<template>
    <v-dialog v-model="dialog" overlay-color="blue" persistent width="800">
        <v-card elevation="3">
            <v-card-title>
                Pay Order: {{ localOrder.code ? localOrder.code.toUpperCase() : '' }}
                <v-chip :color="localOrder.status_colour" class="ma-2" dark small>{{ localOrder.status_text }}</v-chip>
                <v-chip v-show="totalPaid > 0" class="ma-2" color="success" dark small>
                    Paid: {{ totalPaid | number_format }}
                </v-chip>
                <v-chip v-show="totalDiscount > 0" class="ma-2" color="primary" dark small>
                    Disc.:{{ totalDiscount | number_format }}
                </v-chip>
                <v-chip v-show="totalDiscount > 0" class="ma-2" color="info" dark small>
                    Bal.:{{ orderTotal - totalPaid - totalDiscount | number_format }}
                </v-chip>
                <v-spacer/>
                <v-btn icon small @click="close">
                    <v-icon>mdi-close</v-icon>
                </v-btn>
            </v-card-title>
            <v-divider/>
            <v-card-text v-if="this.localOrder && this.localOrder.items && this.localOrder.items.length"
                         class="overflow-y-auto">
                <v-form ref="paymentForm" class="mt-3">
                    <v-row>
                        <v-col class="pb-0" cols="6" md="3">
                            <v-text-field :value="localOrder.customer.name" filled hide-details label="Customer"
                                          readonly/>
                        </v-col>
                        <v-col class="pb-0" cols="6" md="3">
                            <v-text-field v-model="total" filled hide-details label="Balance payable" readonly/>
                        </v-col>
                        <v-col class="pb-0" cols="3" md="3">
                            <v-text-field v-model.number="form.discount" :rules="discountRule" filled
                                          label="Discount"/>
                        </v-col>
                        <v-col class="pb-0" cols="3" md="3">
                            <v-text-field v-model="form.reference" :rules="refRule" filled label="Ref. number"
                                          prepend-inner-icon="mdi-receipt-text-check-outline"/>
                        </v-col>
                        <v-col class="pb-0" cols="3" md="3">
                            <v-text-field v-model="form.amount" :rules="amountRule" filled
                                          label="Amount received">
                                <template #append>
                                    <v-btn icon @click="form.amount = getTotalPayable">
                                        <v-icon>mdi-auto-fix</v-icon>
                                    </v-btn>
                                </template>
                            </v-text-field>
                        </v-col>
                        <v-col class="pb-0" cols="3" md="3">
                            <datepicker :options="{rules: dateRule, filled: true}" label="Date"
                                        @date="form.date = $event"/>
                        </v-col>
                        <v-col class="pb-0" cols="6" md="6">
                            <v-file-input v-model="form.attachment" :rules="attachmentRule"
                                          accept=".gif,.jpg,.jpeg,.png,.pdf"
                                          filled label="Click to attach a justification"
                                          prepend-icon="" prepend-inner-icon="mdi-paperclip"/>
                        </v-col>
                    </v-row>
                    <v-row>
                        <v-col>
                            <v-data-table :headers="headers" :items="localOrder.items">
                                <template #item.index="{index}">
                                    {{ index + 1 }}
                                </template>
                                <template #item.pivot.unit_price="{ item }">
                                    <span>{{ item.pivot.unit_price | number_format }}</span>
                                </template>
                                <template #header.total="{ header }" style="width: 900px">{{ header.text }}</template>
                                <template #item.total="{item}">
                                    {{ item.pivot.quantity * item.pivot.unit_price | number_format }}
                                </template>
                                <template #body.append="{ items }">
                                    <tr class="pink--text">
                                        <th align="center" class="subtitle-1 text-center" colspan="3">Total</th>
                                        <th align="center" class="subtitle-1 text-center">
                                            {{
                                                items.map(i => i.pivot.quantity).reduce((a, b) => a + b, 0) | number_format
                                            }}
                                        </th>
                                        <th class="subtitle-1 text-center">
                                            {{
                                                orderTotal | number_format('FCFA')
                                            }}
                                        </th>
                                    </tr>
                                </template>
                            </v-data-table>
                        </v-col>
                    </v-row>
                </v-form>
            </v-card-text>
            <v-divider/>
            <v-card-actions>
                <v-btn :loading="loading" color="error" outlined plain small @click="close">
                    <v-icon left small>mdi-window-close</v-icon>
                    Close
                </v-btn>
                <v-spacer/>
                <v-btn :loading="loading" color="green accent-4" dark depressed small @click="validatePayment">
                    Validate
                    <v-icon right>mdi-content-save-outline</v-icon>
                </v-btn>
            </v-card-actions>
        </v-card>
    </v-dialog>
</template>

<script>

import OrderItem from "../../../Components/Book/OrderItem.vue";
import Datepicker from "../../../Components/Picker/Datepicker.vue";

export default {
    name: "PayOrderDialog",
    components: {Datepicker, OrderItem},
    props: {
        order: {
            type: Object,
            required: true
        },
        dialog: Boolean
    },
    data() {
        return {
            headers: [
                {
                    text: 'S/N',
                    align: 'center',
                    sortable: false,
                    value: 'index',
                },
                {
                    text: 'Item',
                    align: 'start',
                    sortable: false,
                    value: 'title',
                },
                {
                    text: 'Unit price',
                    align: 'center',
                    sortable: false,
                    value: 'pivot.unit_price',
                },
                {
                    text: 'Qty',
                    align: 'center',
                    sortable: false,
                    value: 'pivot.quantity',
                },
                {
                    text: 'Total',
                    align: 'center',
                    sortable: false,
                    value: 'total',
                    width: '150px'
                },
            ],
            localOrder: {
                customer: {},
                items: [],
                payments: []
            },
            form: {
                amount: null,
                discount: null,
                date: null,
                reference: '',
                attachment: null
            },
            total: null,
            loading: false
        }
    },
    watch: {
        dialog(value) {
            if (value) {
                this.localOrder = Object.assign({}, this.order)
            } else {
                this.searchTerm = ''
                this.selectedItem = {}
                this.searchResults = []
                this.localOrder = {
                    customer: {},
                    items: [],
                    payments: []
                }
                Object.keys(this.form).forEach(v => (this.form[v] = null))
            }
        },
        order(value) {
            if (!Object.keys(value).length) this.close()
        },
        searchTerm(search) {
            search && search !== this.selectedItem?.title && this.searchItems(search)
        },
        getTotalPayable(value) {
            this.total = this.$options.filters.number_format(value, 'XAF')
        }
    },
    computed: {
        orderTotal () {
            return this.localOrder.items.map(i => i.pivot.quantity * i.pivot.unit_price).reduce((a, b) => a + b, 0)
        },
        getTotalPayable() {
            if (!this.localOrder || !this.localOrder.items) return 0
            return this.orderTotal - (typeof this.form.discount === 'number' ? this.form.discount : 0) - this.alreadyPaid
        },
        totalPaid() {
            return this.localOrder.payments.map(p => (p.amount)).reduce((p, acc) => p + acc, 0)
        },
        totalDiscount() {
            return this.localOrder.payments.map(p => (p.discount)).reduce((p, acc) => p + acc, 0)
        },
        alreadyPaid() {
            return this.totalPaid + this.totalDiscount
        },
        amountRule() {
            return [
                v => {
                    if (!v || v > this.getTotalPayable) {
                        return 'Amount cannot be greater than the payable amount.'
                    }
                    return true
                }
            ]
        },
        refRule() {
            return [
                v => {
                    if (!v && !this.form.attachment) {
                        return 'Either provide a reference'
                    }
                    return true
                }
            ]
        },
        attachmentRule() {
            return [
                v => {
                    if (!v && !this.form.reference) {
                        return 'Or provide a attachment as proof of payment'
                    }
                    return true
                }
            ]
        },
        discountRule() {
            return [
                v => {
                    if (v && typeof this.form.discount !== 'number') {
                        return 'Please, enter a valid number'
                    }
                    return true
                }
            ]
        },
        dateRule() {
            return [
                v => {
                    if (!v) {
                        return 'Please, choose a date'
                    }
                    return true
                }
            ]
        }
    },
    methods: {
        validatePayment() {
            if (this.$refs.paymentForm.validate()) {
                this._confirmDialog({
                    message: 'Update this order to PAID?', confirm: async () => {
                        const data = Object.assign({}, this.form)
                        if (data.attachment) {
                            data.attachment = (await this.readAsDataURL(data.attachment)).data
                        }
                        axios.post(this.$route('order.pay', this.localOrder.id), data)
                            .then(resp => resp.data)
                            .then(data => {
                                this.showSnack({message: 'Status updated.'})
                                this.$emit('updated', data)
                                this.close()
                            })
                            .catch(this.errHandler)
                    }
                })
            }
        },
        close() {
            this.$emit('close')
        }
    }
}
</script>

<style scoped>
</style>
